# shellcheck shell=sh
# shellcheck disable=SC2034 
# shellcheck disable=SC2059
# shellcheck disable=SC1091
# shellcheck disable=SC1090
# shellcheck disable=SC2116

trap fo_close INT QUIT TERM EXIT

FO_SCRIPT="$(basename "${0}")"
readonly FO_SCRIPT
readonly RED="\033[1;41m"
readonly NORM="\033[0m"
readonly UNMOUNTED="# "

FO_DISPOSITION="Disposition unavailable"
FO_TEXTILE="Disposition unavailable"
FO_TEXTILE_NOBORDER=""

auxilium_add nolock  -t opt -m "Don\'t create lock file"

another_instance() {
  logger -t "udev" "===> There is another instance running, exiting"
  exit 1
}

fo_repeat_char(){
  # repeat a char X 
	# param: char to repeat
	# param: nb of repeat
	c="$1"
	# shellcheck disable=SC2086
	printf -- "${c}%.0s" $(seq 1 ${2})
}

fo_contain(){
	# check if a string ($1) contain another ($2)
	# param: String to check if contain another
	# param: String to check if is in another
	[ "$#" -lt 2 ] && return 1
	case " $1 " in
		*" ${2} "*)
			echo 1
			;;
		*)
			echo 0
			;;
	esac
}

# Argument 1: The index
# Argument 2: The array
fo_array_get_value() {
	shift "$(($1 +1))" # Shift N number of elements, including the first argument
	# Return non-zero if index is out of bounds ($1 will be empty)
	echo "${1:?Index out of bounds}" # Print the first item after shifting
}

# Argument 1: The value
# Argument 2: The array
fo_array_get_idx() {
	dev=$1
	shift 1
	cpt=0
	total=$#
	while [ "$cpt" -le "$total" ]; do
		if [ "$1" = "$dev" ]; then
			echo "$cpt"
			exit 0
		fi
		cpt=$((cpt + 1))
		shift 1
	done 
	echo "$dev Not found" >&2
}

fo_center_align(){
	# Center align a text
	# Param: Text to align
	# Param: len of final string
	# Param: If is disk to locate
	
	lenTotal="$1"
	locate="$2"
	text="$3"
	[ -z "$text" ] && text='Unknown'
	lenText=${#text}
	lenTotal=$((lenTotal + 2))
	nb_spaces=$((lenTotal - lenText))
	space=' '
	color_begin=''
	color_end=''
	if [ "$locate" = 1 ];then
		color_begin=${RED}
		color_end=${NORM}
	fi
	first_nb=$((nb_spaces/2))
	first_spaces=$(fo_repeat_char "${space}" ${first_nb})
	end_nb=$((nb_spaces - first_nb))
	end_spaces=$(fo_repeat_char "${space}" ${end_nb})
	ret=$(printf -- "%s%s%s" "${first_spaces}" "$text" "${end_spaces}")
	printf -- "${color_begin}%${lenTotal}s${color_end}" "$ret"
}

fo_get_dev_letter(){
	# Print disk letter (a to z) by is index 
	# Param: Disk Index (beginning at 0)
	num=$((97 + ${1}))
	printf "\\$(printf '%03o' ${num})"
}

fo_get_dev_ascii(){
	# Print disk ascii code of last letter of device name
	# Param: letter
	letter=$(printf '%s' "$1" | awk -F 'sd' '{print $2}')
	name_id=$(printf "%d" "'$letter")
	ret=$((name_id - 97))
	echo $ret
}

fo_lock() {
	# Create a lock except if nolock argument
	LOCKDIR=/var/run/lock
	FO_LOCKFILE=${LOCKDIR}/$(basename "${0}")
	readonly FO_LOCKFILE
	if [ -z "$AUXILIUM_ARG_nolock" ] ; then
		logger -t "udev" "===> Claming lock for $0 in ${FO_LOCKFILE}"
		if [ -f "$FO_LOCKFILE" ]; then
			kill -0 "$(cat "$FO_LOCKFILE")" && another_instance
		fi
		echo $$ > "$FO_LOCKFILE"
	fi
}

fo_to_realsize(){
	# Transform storcli report size (TB instead TiB for exemple)
	# to XXX TB (XXX TiB)
	# Get float value
	value=$(echo "$@" | sed 's/[a-zA-Z]*$//')
	# get decimal of value
	value2=$(echo "$value" |cut -d. -f2 | cut -c1-3 | sed 's/^0*//')
	# Transform int value
	value=$(echo "$value" |cut -d. -f1)
	# Get unit
	unit=$(echo "$@" | sed 's/^[0-9 .]*//')
	case $unit in
		P|PB|PiB)
			value=$(( value * 1125899906842624 ))
			[ -n "$value2" ] && value=$(( value + value2 * 1099511627776 ))
			;;
		T|TB|TiB)
			value=$(( value * 1099511627776 ))
			[ -n "$value2" ] && value=$(( value + value2 * 1073741824 ))
			;;
		G|GB|GiB)
			value=$(( value * 1073741824 ))
			[ -n "$value2" ] && value=$(( value + value2 * 1048576 ))
			;;
		M|MB|MiB)
			value=$(( value *1048576 ))
			[ -n "$value2" ] && value=$(( value + value2 * 1024 ))
			;;
		K|KB|KiB)
			value=$(( value * 1024 ))
			[ -n "$value2" ] && value=$(( value + value2 ))
			;;
	*)
			;;
	esac
	echo "$(numfmt --to=si "$value")B ($(numfmt --to=iec-i "$value")B)"
}

fo_use_cache_file() {
	# Test if a cache file is valid, and delete it if invalid
	# Param: Cache File Suffix
	usecache=false
	# read a cache file, if exist and recent
	if [ -f "$1" ]; then
		lastentry=$(stat -c %Y "$1")
		lastvalid=$(date -d "now -5 min" +%s)
		if [ "$lastentry" -gt "$lastvalid" ]; then
			usecache=true
		else
			rm "$1"
		fi
	elif [ -e "$1" ]; then
		rm "$1"
	fi
	if ! $usecache; then
		logger -t "udev" "[Rebuilding cache file]"
		# We should rebuild the cache: exit 1
		return 1
	else
		logger -t "udev" "[Using cache file]"
		# We can re-use the cache: exit 0
		return 0
	fi
}

fo_check_valid(){
	# Check for validity of utils
	v=$(dpkg-query -W lshw|cut -f2)
	dpkg --compare-versions "${v}" ge 02.19 || readonly FO_LSHW_VALID=$?
  v=$(dpkg-query -W smartmontools|cut -f2)
	dpkg --compare-versions "${v}" ge 7.2 || readonly FO_SMARTCTL_VALID=$?
  v=$(dpkg-query -W jq|cut -f2)
	dpkg --compare-versions "${v}" ge 1.5 || readonly FO_JQ_VALID=$?
	v=$(dpkg-query -W storcli|cut -f2)
	dpkg --compare-versions "${v}" ge 7.2508 || dpkg --compare-versions "${v}" eq 1.25.12-1 || readonly FO_STORCLI_VALID=$?
}

# shellcheck disable=SC2120
fo_load_storcli(){
	# Load strocli Data and store it in cache File
	if [ -z "$FO_JQ_VALID" ] && [ -z "$FO_STORCLI_VALID" ] && [ -z "$FO_NO_USE_STORCLI" ] || [ "$1" = "-f" ] ; then
		if ! fo_use_cache_file "${FO_CACHE_FILE_STORCLI}"; then
			fo_ctl=$(mktemp)
			rm "${FO_CACHE_FILE_STORCLI}" 2>/dev/null || true
			${STORCLI} /call show nolog J|jq -r '.["Controllers"]' > "${fo_ctl}"
			fo_raid_card=$(jq -r '.[0]."Response Data"."Product Name"' "${fo_ctl}")
			for i in $(jq -r '.[]."Command Status".Controller' "$fo_ctl"); do
				fo_phyinfo=$(${STORCLI} "/c${i}/pall" show all nolog J|jq -r '.["Controllers"][]["Response Data"]|with_entries(select(.key=="PhyInfo")).PhyInfo' 2>/dev/null||echo null)
				if [ -n "$(jq -r ".[${i}].\"Response Data\"|select(.!=null).\"VD LIST\"|select(.!=null)[]" "${fo_ctl}")" ]; then
								fo_vdinfo=$(${STORCLI} "/c${i}/vall" show all nolog J|jq -r '.["Controllers"][]["Response Data"]|select(.!=null)')
				else
								fo_vdinfo="null"
				fi
				fo_slots=$(${STORCLI} "/c${i}/eall/sall" show all nolog J|jq -r '.["Controllers"][]["Response Data"]|select(.!=null)')
				fo_slots="$fo_slots"$(${STORCLI} "/c${i}/sall" show all nolog J|jq -r '.["Controllers"][]["Response Data"]|select(.!=null)')
				fo_topo=$(jq -r ".[${i}].\"Response Data\"|select(.!=null)" "${fo_ctl}")
				fo_infos="${fo_infos}{\"Controller\":${i},\"PhyInfo\":${fo_phyinfo},\"VD_Info\":${fo_vdinfo},\"Slots_Info\":${fo_slots},\"All_Info\":${fo_topo}},"
			done
			echo "[${fo_infos%?}]" >> "${FO_CACHE_FILE_STORCLI}"
			rm "$fo_ctl"
		fi
		[ -s "${FO_CACHE_FILE_STORCLI}" ] && return 0
	fi
	return 1
}

fo_check_raid(){
	# Check if we can use Storcli 
	fo_load_storcli
	return $?
}

fo_close() {
	# Close application and remove temporaries files
  auxilium_close
	rm "$FO_LOCKFILE" 2> /dev/null || true
}

fo_get_foname(){
  echo "Disk$(echo "$1"|cut -d'/' -f4|awk -F 's' '{print $2}')"
}

fo_write_by_fo_name(){
	# Write an entry for disk unfound
	id=$(fo_get_dev_ascii "$(basename "$1")")
	printf "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" \
		"" \
    "Disk unfound" \
		"" \
		"$1" \
		"Disk${id}" \
		"" \
		"" \
		"Unfound" \
		"Unfound" \
		"Unfound" \
		"" \
		"" \
		"Unfound" >> "$FO_CACHE_DISKS_INFOS"
}

fo_write_disks_infos(){
	# Write disk informations from lshw
	did="$1"
	kname="$2"
	mount_point="$3"
	scsi="$4"
	majmi="$5"
	vendor="$6"
	model="$7"
	serial="$8"
	version="$9"
	shift 9
	size="$1"
	descr="$2"
	eval "$(udevadm info -q property --export "${kname}")"
	fo_name=$(fo_get_sorted_name -f)
	if [ -z "$mount_point" ] || [ "$mount_point" = "/" ]; then
		eval "$(udevadm info -q property --export "${kname}")"
		mount_point="/srv/node/$(fo_get_sorted_name)"
	fi
	printf "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" \
				"${id}" \
				"" \
				"$kname" \
				"$mount_point" \
				"$fo_name" \
				"$scsi" \
				"$majmin" \
				"$vendor" \
				"$model" \
				"$serial" \
				"$version" \
				"$size" \
				"$descr" >> "$FO_CACHE_DISKS_INFOS"
}

fo_write_disk_storcli(){
	type="$1"
	controller="$2"
	eidslt="$3"
	did="$4"
	model="$5"
	dg="$6"
	state="$7"
	size="$8"
	fo_eid=$(echo "$eidslt" | cut -d: -f1)
	fo_slt=$(echo "$eidslt" | cut -d: -f2)
	if [ "$fo_eid" = " " ]; then
		fo_raid_path="/c${controller}/s${fo_slt}"
	else
		fo_raid_path="/c${controller}/e${fo_eid}/s${fo_slt}"
	fi
	# [ "$type" = '-' ] && [ "$dg" != "-" ] && type=$(jq -r ".[\"TOPOLOGY\"]|select(.!=null)[]|select(.DG==$dg)|select(.Arr==\"-\").Type" "${FO_CACHE_FILE_STORCLI}")
	if [ "$type" = 'JBOD' ]; then
		fo_jbod=$(jq -r ".[${controller}].All_Info.\"JBOD LIST\"|select(.!=null)[]|select(.\"EID:Slt\"==\"${eidslt}\")" "${FO_CACHE_FILE_STORCLI}")
		vendor=$(printf "%s\n" "${fo_jbod}"|jq -r '.Vendor')
		id=$(printf "%s\n" "${fo_jbod}"|jq -r '.ID')
		did=$(printf "%s\n" "${fo_jbod}"|jq -r '.DID')
		descr="${type}: ${state}"
	elif [ "$type" = '-' ]; then
		if [ "$dg" != "-" ]; then
			type=$(jq -r ".[${controller}].All_Info.TOPOLOGY|select(.!=null)[]|select(.DG==$dg)|select(.Arr==\"-\").Type" "${FO_CACHE_FILE_STORCLI}")
			did=$dg
			descr="${type}: ${state}"
		else
			descr="${state}"
		fi
	else
		descr="JBOD: ${state}"
	fi
	fo_drive_detailed="$(mktemp)"
	jq -r ".[${controller}].Slots_Info[]|to_entries[]|select(.key==\"Drive ${fo_raid_path} Device attributes\").value|.\"Manufacturer Id\"+\"^\"+.SN+\"^\"+.\"Firmware Revision\"" "${FO_CACHE_FILE_STORCLI}" 2>/dev/null  > "$fo_drive_detailed"
	IFS=^ read -r vendor sn fw < "$fo_drive_detailed"
	rm "$fo_drive_detailed"
	kname=$(fo_get_kname "$did" "$fo_raid_path")
	if [ "$kname" = "Unknown" ]; then
		fo_name=$(fo_get_foname "$fo_raid_path")
		unset mountpoint majmin scsi
	elif  [ ! -b "/dev/$kname" ];then
		kname="/dev/${kname}"
		fo_name=$(fo_get_foname "$fo_raid_path")
		unset mountpoint majmin scsi
	else
		kname="/dev/${kname}"
		eval "$(udevadm info -q property --export "${kname}" 2>/dev/null)"
		fo_name=$(fo_get_sorted_name -f)
		mount_point=$(mount |grep "${kname}" | grep -v /boot/efi | cut -d' ' -f3)
		majmin="${MAJOR}:${MINOR}"
		scsi=$ID_PATH
		if [ -z "$mount_point" ] || [ "$mount_point" = "/" ]; then
			mount_point="/srv/node/$(fo_get_sorted_name)"
		fi
	fi
	# shellcheck disable=SC2086
	vendor=$(echo $vendor)
	# shellcheck disable=SC2086
	model=$(echo $model)
	# shellcheck disable=SC2086
	sn=$(echo $sn)
	[ -z "$model" ] && model=$ID_MODEL
	[ -z "$fw" ] && fw=$ID_REVISION
	printf "%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s\n" \
		"$did" \
		"$fo_raid_path" \
		"$kname" \
		"$mount_point" \
		"$fo_name" \
		"$scsi" \
		"$majmin" \
		"$vendor" \
		"$model" \
		"$sn" \
		"$fw" \
		"$(fo_to_realsize "$size")" \
		"$descr" >> "$FO_CACHE_DISKS_INFOS"
	unset did kname fo_name mount_point scsi majmin model vendor sn fw size descr
}

fo_load_disks_infos(){
	# File format : did,kname,mountpoint,fo_name,scsi,majmin,
	# vendor,model,serial,version,size,description
	#id,kname,mountpoint,scsi,majmin,status,vendor,model,serial,version,size,description
	fo_use_cache_file "${FO_CACHE_DISKS_INFOS}" && return 0
	if fo_check_raid; then
		fo_dl="$(mktemp)"
		# jq -r '.[].Slots_Info[]|select(.!=null)|."PD LIST"[]+{"Controller":.Controller}' "${FO_CACHE_FILE_STORCLI}"  > "$fo_dl" 2>/dev/null || true
		# jq -r '.[].Slots_Info[].|select(.!=null)|."Drive LIST"[]+{"Controller":.Controller}' "${FO_CACHE_FILE_STORCLI}" >> "$fo_dl" 2>/dev/null || true
		jq -r '.[].Controller as $ctl|.[$ctl].Slots_Info+{"Controller":$ctl}' "${FO_CACHE_FILE_STORCLI}"  > "$fo_dl" 2>/dev/null || true
		# jq -r '.[].Slots_Info[].|select(.!=null)|."Drive LIST"[]+{"Controller":.Controller}' "${FO_CACHE_FILE_STORCLI}" >> "$fo_dl" 2>/dev/null || true
		fo_jb="$(mktemp)"
		jq -r '.Controller as $ctl|.[]|to_entries[]|select(.key==0).value|.Type+","+($ctl|tostring)+","+."EID:Slt"+","+(.DID|tostring)+","+.Model+","+(.DG|tostring)+","+.State+","+.Size' "$fo_dl" 2>/dev/null > "$fo_jb" || true
		while IFS=, read -r type controller eidslt did model dg state size;do
			fo_write_disk_storcli "$type" "$controller" "$eidslt" "$did" "$model" "$dg" "$state" "$size"
		done < "$fo_jb"
		rm  "$fo_jb" "$fo_dl" || true
		for i in /dev/sd[a-z]; do
			if ! grep -q "$i" "$FO_CACHE_DISKS_INFOS" && [ -z "$FO_LSHW_VALID" ]; then
				fo_hw=$(mktemp)
				lshw -c disk -json \
				| jq -r ".[]|select(.logicalname==\"${i}\")" \
				| jq -r 'select(.id!="medium")|
				         .id+","+.description+","+.vendor+","+.product+","+.businfo+","+.version+","+.serial+","+(.size|tostring)' > "$fo_hw"
				 IFS=, read -r id descr vendor model businfo fw sn size < "$fo_hw"
				 id=$(echo "$id" | cut -d: -f2)
				 scsi=$(echo "$businfo" | cut -d@ -f2)
				 size=$(fo_to_realsize "$size")
				 majmin=$(lsblk "${i}" -n -J | jq -r '.[][]."maj:min"')
				 fo_write_disks_infos "${id}" "${i}" "" "$scsi" "$majmin" "$vendor" "$model" "$sn" "$fw" "$size" "$descr"
			fi
		done
		unset id scsi majmin vendor model serial version size descr
	else
		fo_use_cache_file "${FO_CACHE_FILE_LSHW}" || lshw -c disk 2>/dev/null > "${FO_CACHE_FILE_LSHW}" || true
		while IFS=: read -r k v; do
			# shellcheck disable=SC2086
			k=$(echo ${k})

			[ -z "$skip" ] || continue
			case "$k" in
			'*-cdrom')
					[ -n "$id" ] && fo_write_disks_infos "${id}" "$kname" "$mount_point" "$scsi" "$majmin" "$vendor" "$model" "$serial" "$version" "$size" "$descr"
					unset id kname mount_point scsi majmin vendor model serial version size descr
					skip=1
			;;
			'*-disk')
				if [ -z "$v" ]; then
					[ -n "$id" ] && fo_write_disks_infos "${id}" "$kname" "$mount_point" "$scsi" "$majmin" "$vendor" "$model" "$serial" "$version" "$size" "$descr"
					unset id kname mount_point scsi majmin vendor model serial version size descr
					skip=1
				else
					[ -n "$id" ] && fo_write_disks_infos "${id}" "$kname" "$mount_point" "$scsi" "$majmin" "$vendor" "$model" "$serial" "$version" "$size" "$descr"
					unset skip kname id mount_point scsi majmin vendor model serial version size descr
					id="${v}"
				fi
				;;
			'description')
				# shellcheck disable=SC2086
				descr=$(echo ${v})
			;;
			'logical name')
				case "$v" in
				" /dev/"*)
					# shellcheck disable=SC2086
					kname=$(echo ${v})
					;;
				" /srv/node/"*)
					# shellcheck disable=SC2086
					mount_point=$(echo ${v})
					;;
				esac
			;;
			'vendor')
				# shellcheck disable=SC2086
				vendor=$(echo ${v})
			;;
			'product')
				# shellcheck disable=SC2086
				model=$(echo ${v})
			;;
			'bus info')
				scsi=$(echo "${v}" | cut -d@ -f2)
			;;
			'version')
				# shellcheck disable=SC2086
				version=$(echo ${v})
			;;
			'serial')
				# shellcheck disable=SC2086
				serial=$(echo ${v})
			;;
			'size')
				# shellcheck disable=SC2086
				size=$(echo ${v})
			;;
			esac
		done < "${FO_CACHE_FILE_LSHW}"
		[ -z "$skip" ] && fo_write_disks_infos "$id" "$kname" "$mount_point" "$scsi" "$majmin" "$vendor" "$model" "$serial" "$version" "$size" "$descr"
		printf "" >&2
	fi
}

fo_get_disk_info(){
	field="$2"
	search=",${1},"
	grep "${search}" "${FO_CACHE_DISKS_INFOS}" > /tmp/fo_grp
	while IFS=, read -r id raid_path kname mount_point fo_name scsi majmin vendor model serial fw size descr; do
		case "$field" in
		id)
			echo "${id}"
		;;
		raid_path)
			echo "$raid_path"
		;;
		majmin)
			echo "${majmin}"
		;;
		scsi)
			echo "${scsi}"
		;;
		vendor)
			echo "${vendor}"
		;;
		serial)
			echo "${serial}"
		;;
		model)
			echo "${model}"
		;;
		firmware)
			echo "${fw}"
		;;	
		description)
			echo "${descr}"
		;;
		mount_point)
			echo "${mount_point}"
		;;
		fo_name)
			echo "${fo_name}"
		;;
		kname)
			echo "${kname}"
		;;
		size)
			echo "${size}"
		;;
		esac
	done < /tmp/fo_grp
	rm /tmp/fo_grp
}

fo_get_raid_path(){
	for ctl in $(${STORCLI} /call show nolog J|jq -r '.Controllers[]."Command Status".Controller');do
		vd=$(jq ".[${ctl}].VD_Info|select(.!=null)|to_entries[]
												|select(.key|tostring|endswith(\"Properties\"))
													|select(.value.\"SCSI NAA Id\"==\"${1}\").key" \
										"${FO_CACHE_FILE_STORCLI}"| \
										cut -d' ' -f1 | \
										awk -F 'VD' '{print $2}')
		if [ -n "${vd}" ]; then
			eid_slt=$(jq -r ".[${ctl}].VD_Info.\"PDs for VD ${vd}\"|select(.!=null)[].\"EID:Slt\"" "${FO_CACHE_FILE_STORCLI}")
			eid=$(echo "$eid_slt" |cut -d: -f1)
			slt=$(echo "$eid_slt" |cut -d: -f2)
			echo "/c${ctl}/e${eid}/s${slt}"
			break
		fi
	done
}

fo_get_jbod_path(){
  jq -r ".[].Slots_Info[]|to_entries[]|select(.key|tostring|endswith(\"attributes\"))|select(.value.SN==\"${1}\").key" "${FO_CACHE_FILE_STORCLI}" | cut -d' ' -f2
}

fo_get_storcli_path(){
  if [ "$(echo "$ID_MODEL"|cut -d_ -f1)" = "RAID" ]; then
		fo_get_raid_path "$ID_SERIAL_SHORT"
	else
		fo_get_jbod_path "$ID_SCSI_SERIAL"
	fi
}

fo_get_serial_by_vd(){
	jq -r ".[].Slots_Info.\"Drive ${1} - Detailed Information\"|select(.!=null).\"Drive ${1} Device attributes\".SN" "${FO_CACHE_FILE_STORCLI}"
}

fo_get_serial(){
  if [ "$(echo "$ID_MODEL"|cut -d_ -f1)" = "RAID" ]; then
		raidPath=$(fo_get_raid_path "$ID_SERIAL_SHORT")
		fo_get_serial_by_vd "$raidPath"
	else
		echo "$ID_SCSI_SERIAL"
	fi
}

fo_get_kname_by_serial(){
 jq -r ".[][]|select(.serial==\"${1}\").name" "${FO_CACHE_FILE_LSBLK}"
}

fo_init(){
	if [ -f "/etc/ferreus-orbit/ferreus-orbit.conf" ] ; then
		. "/etc/ferreus-orbit/ferreus-orbit.conf"
	elif [ -f "etc/ferreus-orbit.conf" ] ; then
		. "etc/ferreus-orbit.conf"
	else
		echo "Error: could not load ferreus-orbit.conf"
		exit 1
	fi

	FO_DEFAULT_RAID0=0

	readonly LSHW=/usr/bin/lshw
	readonly LSBLK=/usr/bin/lsblk
	readonly VAR_PATH='/var/cache/ferreus-orbit/'
	mkdir "$VAR_PATH" 2>/dev/null|| true
	
	readonly FO_CACHE_FILE_LSHW="${VAR_PATH}fo_cache_lshw"
	readonly FO_CACHE_FILE_LSBLK="${VAR_PATH}fo_cache_lsblk"
	readonly FO_CACHE_FILE_STORCLI="${VAR_PATH}fo_cache_storcli"
	readonly FO_CACHE_DISKS_INFOS="${VAR_PATH}fo_disks_infos"
	readonly FO_CACHE_WWN="${VAR_PATH}fo_wwn"
	if [ "$1" = "clean" ]; then
		rm -f "${FO_CACHE_FILE_LSHW}"|| true
		rm -f "${FO_CACHE_FILE_LSBLK}"|| true
		rm -f "${FO_CACHE_FILE_STORCLI}"|| true
		rm -f "${FO_CACHE_DISKS_INFOS}"|| true
		rm -f "${FO_CACHE_WWN}"|| true
	fi

	if [ -x /usr/bin/storcli64 ] ; then
		readonly STORCLI=/usr/bin/storcli64
	elif [ -x /usr/bin/storcli_x64 ] ; then
		readonly STORCLI=/usr/bin/storcli_x64
	elif [ -x /usr/bin/storcli ] ; then
		readonly STORCLI=/usr/bin/storcli
	fi
	readonly FO_ETC_PLUGINS_PATH="/etc/ferreus-orbit/plugins"
	if [ -d "/usr/share/ferreus-orbit/plugins" ]; then
		readonly FO_PLUGINS_PATH="/usr/share/ferreus-orbit/plugins"
	elif [ -d "share/plugins" ] ; then
		readonly FO_PLUGINS_PATH="share/plugins"
	else
		readonly FO_PLUGINS_PATH="../share/plugins"
	fi

	FO_CHASSIS_MANUFACTURER=$(dmidecode -s system-manufacturer)
	readonly FO_CHASSIS_MANUFACTURER
	FO_CHASSIS_MODEL=$(dmidecode -s system-product-name)
	readonly FO_CHASSIS_MODEL


	fo_check_valid
	if [ -d /dev/disk/oci-sort ] ; then
		USE_FO_NAME=0
		readonly SORTED_PATH='/dev/disk/oci-sort'
	elif [ -d /dev/disk/fo-sort ] ; then
		USE_FO_NAME=1
		readonly SORTED_PATH='/dev/disk/fo-sort'
	else
		USE_FO_NAME=-1
		readonly SORTED_PATH='/dev'
	fi
	if [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
	elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/${FO_CHASSIS_MODEL}"
	elif [ -f "${FO_ETC_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
	elif [ -f "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic" ]; then
		. "${FO_PLUGINS_PATH}/${FO_CHASSIS_MANUFACTURER}/generic"
	else
		. "${FO_PLUGINS_PATH}/generic"
	fi
	[ -f "${LSBLK}" ] && ${LSBLK} -o name,path,mountpoint,serial,fssize --json > "${FO_CACHE_FILE_LSBLK}"
	fo_load_disks_infos
}
